@@ -4,7 +4,6 @@ import numpy as np
 import sys
 import logging
 from methplotlib.utils import file_sniffer
-import pysam
 
 
 class Methylation(object):
@@ -15,6 +14,22 @@ class Methylation(object):
         self.called_sites = called_sites
 
 
+def get_data(methylation_files, names, window, smoothen=5):
+    """
+    Import methylation data from all files in the list methylation_files
+
+    Data can in various formats
+    - nanopolish raw, phased, frequency
+    - cram
+    - nanocompore
+    - bedgraph
+
+    data is extracted within the window args.window
+    Frequencies are smoothened using a sliding window
+    """
+    return [read_meth(f, n, window, smoothen) for f, n in zip(methylation_files, names)]
+
+
 def read_meth(filename, name, window, smoothen=5):
     """
     converts a file from nanopolish to a pandas dataframe
@@ -26,7 +41,7 @@ def read_meth(filename, name, window, smoothen=5):
     which will return a dataframe with 'read', 'chromosome', 'pos', 'log_lik_ratio', 'strand'
     """
     file_type = file_sniffer(filename)
-    logging.info("File is of type {}".format(file_type))
+    logging.info(f"File {filename} is of type {file_type}")
     try:
         if file_type.startswith("nanopolish"):
             return parse_nanopolish(filename, file_type, name, window, smoothen=smoothen)
@@ -34,14 +49,48 @@ def read_meth(filename, name, window, smoothen=5):
             return parse_nanocompore(filename, name, window)
         elif file_type == "ont-cram":
             return parse_ont_cram(filename, name, window)
-    except Exception:
-        sys.stderr.write("\n\n\nInput file {} not recognized!\n".format(filename))
+        elif file_type == 'bedgraph':
+            return parse_bedgraph(filename, name, window)
+    except Exception as e:
+        logging.error(f"Error processing {filename}.")
+        logging.error(e, exc_info=True)
+        sys.stderr.write(f"\n\n\nError processing {filename}!\n")
         sys.stderr.write("\n\n\nDetailed error:\n")
         raise
 
 
 def parse_nanopolish(filename, file_type, name, window, smoothen=5):
-    table = pd.read_csv(filename, sep="\t")
+    if window:
+        from pathlib import Path
+        if Path(filename + '.tbi').is_file():
+            import subprocess
+            import gzip
+            logging.info(f"Reading {filename} using a tabix stream.")
+            region = f"{window.chromosome}:{window.begin}-{window.end}"
+            try:
+                tabix_stream = subprocess.Popen(['tabix', filename, region],
+                                                stdout=subprocess.PIPE,
+                                                stderr=subprocess.PIPE)
+            except FileNotFoundError as e:
+                logging.error("Error when opening a tabix stream.")
+                logging.error(e, exc_info=True)
+                sys.stderr.write("\n\nERROR when opening a tabix stream.\n")
+                sys.stderr.write("Is tabix installed and on the PATH?.")
+                raise
+            header = gzip.open(filename, 'rt').readline().rstrip().split('\t')
+            table = pd.read_csv(tabix_stream.stdout, sep='\t', header=None, names=header)
+        else:
+            logging.info(f"Reading {filename} slowly by splitting the file in chunks.")
+            sys.stderr.write(f"\nReading {filename} would be faster with bgzip and tabix.\n")
+            if file_type in ['nanopolish_call', 'nanopolish_phased']:
+                sys.stderr.write("Please index with 'tabix -S1 -s1 -b3 -e4'.\n")
+            else:
+                sys.stderr.write("Please index with 'tabix -S1 -s1 -b2 -e3'.\n")
+            iter_csv = pd.read_csv(filename, sep="\t", iterator=True, chunksize=1e6)
+            table = pd.concat([chunk[chunk['chromosome'] == window.chromosome]
+                               for chunk in iter_csv])
+    else:
+        table = pd.read_csv(filename, sep="\t")
     gr = pr.PyRanges(table.rename(columns={"start": "Start", "chromosome": "Chromosome",
                                            "end": "End", "Strand": "strand"}))
     logging.info("Read the file in a dataframe.")
@@ -51,7 +100,7 @@ def parse_nanopolish(filename, file_type, name, window, smoothen=5):
     try:
         gr.pos = np.floor(gr.drop().df[["Start", "End"]].mean(axis=1))
     except KeyError:
-        sys.stderr.write("\n\n\nProblem parsing nanopolish file {}!\n".format(filename))
+        sys.stderr.write(f"\n\n\nProblem parsing nanopolish file {filename}!\n")
         sys.stderr.write("Could it be that there are no calls in your selected window?\n")
         sys.stderr.write("\n\n\nDetailed error:\n")
         raise
@@ -105,7 +154,51 @@ def parse_nanocompore(filename, name, window):
         called_sites=len(table))
 
 
+def parse_bedgraph(filename, name, window):
+    if window:
+        from pathlib import Path
+        if Path(filename + '.tbi').is_file():
+            import subprocess
+            logging.info(f"Reading {filename} using a tabix stream.")
+            region = f"{window.chromosome}:{window.begin}-{window.end}"
+            try:
+                tabix_stream = subprocess.Popen(['tabix', filename, region],
+                                                stdout=subprocess.PIPE,
+                                                stderr=subprocess.PIPE)
+            except FileNotFoundError as e:
+                logging.error("Error when opening a tabix stream.")
+                logging.error(e, exc_info=True)
+                sys.stderr.write("\n\nERROR when opening a tabix stream.\n")
+                sys.stderr.write("Is tabix installed and on the PATH?.")
+                raise
+            table = pd.read_csv(tabix_stream.stdout, sep='\t', header=None,
+                                names=['Chromosome', 'Start', 'End', 'Value'])
+        else:
+            logging.info(f"Reading {filename} slowly by splitting the file in chunks.")
+            sys.stderr.write(
+                f"\nReading {filename} would be faster with bgzip and 'tabix -p bed'.\n")
+            iter_csv = pd.read_csv(filename, sep="\t", iterator=True, chunksize=1e6, header=None,
+                                   names=['Chromosome', 'Start', 'End', 'Value'])
+            table = pd.concat([chunk[chunk['Chromosome'] == window.chromosome]
+                               for chunk in iter_csv])
+    else:
+        table = pd.read_csv(filename, sep="\t", header=None,
+                            names=['Chromosome', 'Start', 'End', 'Value'])
+    gr = pr.PyRanges(table)
+    logging.info("Read the file in a dataframe.")
+    if window:
+        gr = gr[window.chromosome, window.begin:window.end]
+        if len(gr.df) == 0:
+            sys.exit(f"No records for {filename} in {window.string}!\n")
+    return Methylation(
+        table=gr.df.sort_values('Start'),
+        data_type="bedgraph",
+        name=name,
+        called_sites=len(table))
+
+
 def parse_ont_cram(filename, name, window):
+    import pysam
     cram = pysam.AlignmentFile(filename, "rc")
     data = []
     for read in cram.fetch(reference=window.chromosome, start=window.begin, end=window.end):
@@ -158,15 +251,3 @@ def errs_tab(n):
 
 def phred_to_probability(quals, tab=errs_tab(128)):
     return [tab[ord(q) - 33] for q in quals]
-
-
-def get_data(methylation_files, names, window, smoothen=5):
-    """
-    Import methylation data from all files in the list methylation_files
-
-    Data can be either frequency or raw.
-
-    data is extracted within the window args.window
-    Frequencies are smoothened using a sliding window
-    """
-    return [read_meth(f, n, window, smoothen) for f, n in zip(methylation_files, names)]
