@@ -89,11 +89,9 @@ def bed_annotation(bed, window):
             for (begin, end, name) in parse_bed(bed, window)]
 
 
-def methylation(meth_data, dotsize=4):
+def methylation(meth_data, dotsize=4, binary=False):
     """
-    Call function get_data to parse files from nanopolish,
-     either the methylation calls (nanopolish_call) or those from calculate_methylation_frequency
-    Return per dataset a list of one (if frequency) or lots of (if nanopolish_call) plotly traces
+    Plot methylation traces from various data types
     """
     traces = []
     types = []
@@ -104,7 +102,8 @@ def methylation(meth_data, dotsize=4):
             traces.append(
                 make_per_read_meth_traces_llr(table=meth.table,
                                               phased=meth.data_type == 'nanopolish_phased',
-                                              dotsize=dotsize)
+                                              dotsize=dotsize,
+                                              binary=binary)
             )
             split = True
         elif meth.data_type == 'nanocompore':
@@ -122,8 +121,18 @@ def methylation(meth_data, dotsize=4):
                                                 dotsize=dotsize)
             )
             split = True
+        elif meth.data_type == 'bedgraph':
+            starts = meth.table["Start"]
+            ends = meth.table["End"]
+            vals = meth.table["Value"]
+            traces.append(
+                [go.Scatter(x=[val for pair in zip(starts, ends) for val in pair],
+                            y=[val for pair in zip(vals, vals) for val in pair],
+                            mode='lines',
+                            name=meth.name,
+                            hoverinfo='name')])
         else:
-            sys.exit("ERROR: unrecognized data type {}".format(meth.data_type))
+            sys.exit(f"ERROR: unrecognized data type {meth.data_type}")
         types.append(meth.data_type)
         names.append(meth.name)
     return DataTraces(traces=traces,
@@ -132,13 +141,13 @@ def methylation(meth_data, dotsize=4):
                       split=split)
 
 
-def make_per_read_meth_traces_phred(table, max_coverage=100, dotsize=4):
+def make_per_read_meth_traces_phred(table, max_cov=100, dotsize=4):
     """Make traces for each read"""
     minmax_table = find_min_and_max_pos_per_read(table)
-    df_heights = assign_y_height_per_read(minmax_table, max_coverage=max_coverage)
+    df_heights = assign_y_height_per_read(minmax_table, max_coverage=max_cov)
     table = table.join(df_heights, on="read_name")
     traces = []
-    hidden_reads = 0
+    hidden = 0
     for read in table["read_name"].unique():
         strand = table.loc[table["read_name"] == read, "strand"].values[0]
         try:
@@ -148,25 +157,27 @@ def make_per_read_meth_traces_phred(table, max_coverage=100, dotsize=4):
                                          strand=strand)
             )
         except KeyError:
-            hidden_reads += 1
+            hidden += 1
             continue
-    if hidden_reads:
-        sys.stderr.write("Warning: hiding {} reads because coverage above {}x.\n".format(
-            hidden_reads, max_coverage))
+    if hidden:
+        sys.stderr.write(f"Warning: hiding {hidden} reads because coverage above {max_cov}x.\n")
     traces.append(
         make_per_position_phred_scatter(read_table=table, dotsize=dotsize)
     )
     return traces
 
 
-def make_per_read_meth_traces_llr(table, phased=False, max_coverage=100, dotsize=4):
+def make_per_read_meth_traces_llr(table, phased=False, max_cov=100, dotsize=4, binary=False):
     """Make traces for each read"""
     minmax_table = find_min_and_max_pos_per_read(table, phased=phased)
-    df_heights = assign_y_height_per_read(minmax_table, phased=phased, max_coverage=max_coverage)
+    df_heights = assign_y_height_per_read(minmax_table, phased=phased, max_coverage=max_cov)
     table = table.join(df_heights, on="read_name")
-    table.loc[:, "llr_scaled"] = rescale_log_likelihood_ratio(table["log_lik_ratio"].copy())
+    if binary:
+        table.loc[:, "llr_scaled"] = binarize_log_likelihood_ratio(table["log_lik_ratio"].copy())
+    else:
+        table.loc[:, "llr_scaled"] = rescale_log_likelihood_ratio(table["log_lik_ratio"].copy())
     traces = []
-    hidden_reads = 0
+    hidden = 0
     for read in table["read_name"].unique():
         strand = table.loc[table["read_name"] == read, "strand"].values[0]
         if phased:
@@ -181,11 +192,10 @@ def make_per_read_meth_traces_llr(table, phased=False, max_coverage=100, dotsize
                                          phase=phase)
             )
         except KeyError:
-            hidden_reads += 1
+            hidden += 1
             continue
-    if hidden_reads:
-        sys.stderr.write("Warning: hiding {} reads because coverage above {}x.\n".format(
-            hidden_reads, max_coverage))
+    if hidden:
+        sys.stderr.write(f"Warning: hiding {hidden} reads because coverage above {max_cov}x.\n")
     traces.append(
         make_per_position_likelihood_scatter(read_table=table, dotsize=dotsize)
     )
@@ -256,6 +266,19 @@ def rescale_log_likelihood_ratio(llr):
     return llr
 
 
+def binarize_log_likelihood_ratio(llr, cutoff=2):
+    '''
+    Converts the log likelihood to sorta binary representation,
+    with 0 being insufficient evidence (based on cutoff)
+    and +1 being probably methylated
+    and -1 being probably unmethylated
+    '''
+    llr[(-cutoff < llr) & (llr < cutoff)] = 0
+    llr[llr > cutoff] = 1
+    llr[llr < -cutoff] = -1
+    return llr
+
+
 def make_per_read_line_trace(read_range, y_pos, strand, phase=None):
     """Make a grey line trace for a single read,
     with black arrow symbols on the edges indicating strand"""
